
/* @(#)$Id: save_opts.h,v 1.2 1998/10/20 19:57:35 hurtta Exp $ */

/******************************************************************************
 *  The Elm (ME+) Mail System  -  $Revision: 1.2 $   $State: Exp $
 *
 *  Modified by: Kari Hurtta <hurtta+elm@ozone.FMI.FI>
 ******************************************************************************
 *  The Elm Mail System 
 *
 * 			Copyright (c) 1988-1992 USENET Community Trust
 * 			Copyright (c) 1986,1987 Dave Taylor
 *****************************************************************************/

/*
 *	Defines for the storage of options portion of the Elm system.
 */

typedef struct {
    char letter;		/* menu letter on options screen */
    char *menu;			/* menu prompt */
    int  menu_msg;		/* NLS message number of menu prompt */
    char *parm;			/* parameter to modify */
    int (*post)();		/* post processing function */
    char *one_liner;		/* one line help message */
    int  one_liner_msg; 	/* NLS message number of one line help message */
    } opts_menu;

#define DT_SYN 0 /* synonym entry (old name) */
#define DT_STR 1 /* string */
#define DT_NUM 2 /* number */
#define DT_BOL 3 /* ON/OFF (boolean) */
#define DT_CHR 4 /* character */
#define DT_WEE 5 /* weed list */
#define DT_ALT 6 /* alternate addresses list */
#define DT_SRT 7 /* sort-by code */
#define DT_MLT 8 /* multiple destinations for data */
#define DT_ASR 9 /* sort-by code */
#define DT_PRM 10 /* file permissions */
#define DT_MASK 037 /* mask for data type */
#define FL_LOCAL 0040          /* flag if changed */
#define FL_NOSPC 0100          /* flag if preserve blanks as "_" */
#define FL_SYS   0200          /* flag if only valid in system RC */
#define FL_OR    0400          /* flag if boolean value may have been set */
#define FL_AND  01000          /* flag if boolean value may have been unset */

typedef struct {
   int nlen;
   char **list;
}  enumerate_list;

typedef struct { 
	char 	name[NLEN]; 	/* name of instruction */
	long 	offset;		/* offset into elmrc-info file */
        int	flags;	/* DT_STR, DT_NUM, DT_BOL, etc */
        union {
          char 	*str;
          int 	*num;
          int 	*bol;
          char 	*chr;
          char 	**weed;
          struct addr_rec **alts;
          int 	*sort;
          } val;
          int size_val;
          enumerate_list   *e_ptr;
	} save_info_recs;

/*
 *	since many C compilers cannot init a union as a static
 *	init, make the same structure with just the char * for
 *	the union pointer.
 */
typedef struct { 
	char 	name[NLEN]; 	/* name of instruction */
	long 	offset;		/* offset into elmrc-info file */
        int	flags;	/* DT_STR, DT_NUM, DT_BOL, etc */
        char 	*str;
        int              size_val;
        enumerate_list   *e_ptr;
	} save_info_recs_init;

#define SAVE_INFO_STR(x) (save_info[x].val.str)
#define SAVE_INFO_NUM(x) (save_info[x].val.num)
#define SAVE_INFO_BOL(x) (save_info[x].val.bol)
#define SAVE_INFO_CHR(x) (save_info[x].val.chr)
#define SAVE_INFO_WEE(x) (save_info[x].val.weed)
#define SAVE_INFO_ALT(x) (save_info[x].val.alts)
#define SAVE_INFO_SRT(x) (save_info[x].val.sort)
#define SAVE_INFO_ASR(x) (save_info[x].val.sort)
#define SAVE_INFO_SYN(x) (save_info[x].val.str)
#define SAVE_INFO_MLT(x) (save_info[x].val.weed)

#ifdef SAVE_OPTS

/* "lists" for DT_MLT.  These and DT_SYN could be eliminated if support
   of the old parameter names was dropped.
*/
char *SIGS[]={"remotesignature","localsignature",NULL},
	*ALWAYS[]={"alwayskeep","alwaysstore",NULL};

static char * USERLEVELS[] = { "beginner", "intermediate", "expert", NULL };
static enumerate_list USERLEVEL = {3, &(USERLEVELS[0]) };

static char * NOENCODINGS[] = { "pass-7bit", "pass-8bit", "pass-binary", NULL };
static enumerate_list NOENCODING = {3, &(NOENCODINGS[0]) };

save_info_recs_init save_info_data[] = {
{"aliassortby",		-1L,DT_ASR,(char *)&alias_sortby, 0, NULL},
{"alteditor",		-1L,DT_STR,alternative_editor,
   sizeof alternative_editor, NULL},
{"alternatives",	-1L,DT_ALT,(char *)&alternative_addresses, 0, NULL},
{"alwaysdelete",	-1L,DT_BOL,(char *)&always_del, 0, NULL},
{"alwayskeep",		-1L,DT_BOL,(char *)&always_keep, 0, NULL},
{"alwaysleave",		-1L,DT_MLT,(char *)ALWAYS, 0, NULL},
{"alwaysstore",		-1L,DT_BOL,(char *)&always_store, 0, NULL},
{"arrow",		-1L,DT_BOL|FL_OR,(char *)&arrow_cursor, 0, NULL},
{"ask",			-1L,DT_BOL,(char *)&question_me, 0,   NULL},
{"askcc",		-1L,DT_BOL,(char *)&prompt_for_cc, 0, NULL},
#ifdef USE_PGP
{"askpgpsig",		-1L,DT_BOL,(char *)&pgp_askpgpsig, 0, NULL},
#endif
{"attribution",		-1L,DT_STR,attribution, sizeof attribution, NULL},
{"auto-cc",		-1L,DT_SYN,"copy", 0, NULL},
{"autocopy",		-1L,DT_BOL,(char *)&auto_copy, 0, NULL},
#ifdef BACKGROUD_PROCESSES       
{ "background-wait-time",-1L,DT_NUM,(char *)&background_wait_time, 0, NULL }, 
#endif
  /* {"bounce",		-1L,DT_SYN,"bounceback", 0, NULL},
     {"bounceback",		-1L,DT_NUM,(char *)&bounceback, 0, NULL},
     */
{"builtinlines",	-1L,DT_NUM,(char *)&builtin_lines, 0, NULL},
{"calendar",		-1L,DT_STR,raw_calendar_file, 
   sizeof raw_calendar_file, NULL},
{"cc",			-1L,DT_SYN,"askcc", 0, NULL},
#ifdef MIME
{"charset",		-1L,DT_STR,charset, sizeof charset, NULL},
{"compatcharsets",		-1L,DT_STR,charset_compatlist,
   sizeof charset_compatlist, NULL},
#endif
{"configoptions",	-1L,DT_STR,config_options, sizeof config_options, 
   NULL},
{"confirmappend",	-1L,DT_BOL,(char *)&confirm_append, 0, NULL},
{"confirmcreate",	-1L,DT_BOL,(char *)&confirm_create, 0, NULL},
{"confirmfiles",	-1L,DT_BOL,(char *)&confirm_files, 0, NULL},
{"confirmfolders",	-1L,DT_BOL,(char *)&confirm_folders, 0, NULL},
{"copy",		-1L,DT_BOL,(char *)&auto_cc, 0, NULL},
{"delete",		-1L,DT_SYN,"alwaysdelete",   0, NULL},
#ifdef MIME
{"displaycharset",	-1L,DT_STR,raw_display_charset,
   sizeof raw_display_charset, NULL},
#endif
#ifdef USE_DSN
{"dsn-success",         -1L,DT_BOL,(char *)&DSN_success, 0, NULL },
#endif
{"easyeditor",		-1L,DT_STR,e_editor,   sizeof e_editor,   NULL},
{"editor",		-1L,DT_STR,raw_editor, sizeof raw_editor, NULL},
{"escape",		-1L,DT_CHR,(char *)&escape_char, 0,       NULL},
{"folders",		-1L,DT_SYN,"maildir", 0,                  NULL},
{"forcename",		-1L,DT_BOL,(char *)&force_name, 0,        NULL},
{"form",		-1L,DT_SYN,"forms", 0,                    NULL},
{"forms",		-1L,DT_BOL,(char *)&allow_forms, 0,       NULL},
{"fullname",		-1L,DT_STR,full_username, sizeof full_username, NULL},
{"hostdomain",		-1L,DT_STR|FL_SYS,hostdomain,   sizeof hostdomain, 
   NULL},
{"hostfullname",	-1L,DT_STR|FL_SYS,hostfullname, sizeof hostfullname, 
   NULL},
{"hostname",		-1L,DT_STR|FL_SYS,hostname,     sizeof hostname, 
   NULL},
  /* {"hpkeypad",		-1L,DT_SYN,"keypad", 0, NULL}, */
  /* {"hpsoftkeys",		-1L,DT_SYN,"softkeys", 0, NULL}, */
{"keep",		-1L,DT_SYN,"keepempty", 0, NULL},
{"keepempty",		-1L,DT_BOL,(char *)&keep_empty_files, 0, NULL},
#ifdef USE_PGP
{"keeppassfor", -1L,DT_NUM,(char *)&pgp_keeppassfor, 0, NULL},
#endif
  /* {"keypad",		-1L,DT_BOL|FL_OR,(char *)&hp_terminal, 0, NULL}, */
{"localsignature",	-1L,DT_STR,raw_local_signature, 
   sizeof raw_local_signature, NULL},
{"lock-in-copy",        -1L,DT_BOL,(char *)&lock_in_copy,0, NULL},
{"mailbox",		-1L,DT_SYN,"receivedmail", 0, NULL},
{"maildir",		-1L,DT_STR,raw_folders, sizeof raw_folders, NULL},
{"mailedit",		-1L,DT_SYN,"editor", 0, NULL},
{"mailpermissions",	-1L,DT_PRM,(char *)&mail_permissions, 0, NULL},
{"menu",		-1L,DT_BOL|FL_AND,(char *)&mini_menu, 0, NULL},
{"menus",		-1L,DT_SYN,"menu", 0, NULL},
#ifdef MIME
{"metamail",		-1L,DT_STR,raw_metamail_path, 
   sizeof raw_metamail_path, NULL},
#endif
{"metoo",		-1L,DT_BOL,(char *)&metoo, 0, NULL},
#ifdef MIME
{"mimeforward",		-1L,DT_BOL,(char *)&mimeforward, 0, NULL},
#endif
{"movepage",		-1L,DT_BOL,(char *)&move_when_paged, 0, NULL},
{"movewhenpaged",	-1L,DT_SYN,"movepage", 0, NULL},
{"name",		-1L,DT_SYN,"fullname", 0, NULL},
{"names",		-1L,DT_BOL,(char *)&names_only, 0, NULL},
#ifdef MIME
{"noencoding",-1L,DT_NUM,(char *)&allow_no_encoding, 0, &NOENCODING},
                                 /* 1 (8bit): Allow 8bit without -B8BITMIME
                                  * 2 (biary): Allow binary without -BBINARYMIME and
                                  *    and 8bit without -B8BITMIME */
{"nohdrencoding",       -1L,DT_BOL,(char *)&allow_no_hdrencoding, 0, NULL},
#endif /* MIME **/
{"noheader",		-1L,DT_BOL,(char *)&noheader, 0, NULL},
{"noheaderfwd",		-1L,DT_BOL,(char *)&noheaderfwd, 0, NULL},
{"page",		-1L,DT_SYN,"pager", 0, NULL},
#ifdef MIME
{"pagealternative",	-1L,DT_BOL,(char *)&pagealternative, 0, NULL},
{"pagemultipart",	-1L,DT_BOL,(char *)&pagemultipart, 0, NULL},
#endif /* MIME */
{"pager",		-1L,DT_STR,raw_pager, sizeof raw_pager, NULL},
#ifdef USE_PGP
{"pgp",		        -1L,DT_STR,raw_pgp_path, sizeof raw_pgp_path, NULL},
#endif
{"pointnew",		-1L,DT_BOL,(char *)&point_to_new, 0, NULL},
{"pointtonew",		-1L,DT_SYN,"pointnew", 0, NULL},
{"precedences",		-1L,DT_STR,allowed_precedences, 
   sizeof allowed_precedences, NULL},
{"prefix",		-1L,DT_STR|FL_NOSPC,prefixchars,
   sizeof prefixchars, NULL},
{"print",		-1L,DT_STR,raw_printout, sizeof raw_printout, NULL},
{"printmail",		-1L,DT_SYN,"print", 0, NULL},
{"promptafter",		-1L,DT_BOL,(char *)&prompt_after_pager, 0, NULL},
{"question",		-1L,DT_SYN,"ask", 0, NULL},
{"quoteforward",	-1L,DT_BOL,(char *)&quote_forward, 0, NULL},
{"readmsginc",		-1L,DT_NUM,(char *)&readmsginc, 0, NULL},
{"receivedmail",	-1L,DT_STR,raw_recvdmail, sizeof raw_recvdmail, NULL},
{"remotesignature",	-1L,DT_STR,raw_remote_signature, 
   sizeof raw_remote_signature, NULL},
#ifdef MIME
{"require-mime-version-for-body-encoding",
   -1L,DT_BOL,(char *)&req_mime_bodyencoding, 0, NULL},
{"require-mime-version-for-hdr-encoding",
   -1L,DT_BOL,(char *)&req_mime_hdrencoding, 0, NULL},
#endif
{"resolve",		-1L,DT_BOL,(char *)&resolve_mode, 0, NULL},
{"savebyname",		-1L,DT_SYN,"savename", 0, NULL},
{"savemail",		-1L,DT_SYN,"sentmail", 0, NULL},
{"savename",		-1L,DT_BOL,(char *)&save_by_name, 0, NULL},
{"saveto",		-1L,DT_SYN,"sentmail", 0, NULL},
{"sentmail",		-1L,DT_STR,raw_sentmail, sizeof raw_sentmail, NULL},
{"shell",		-1L,DT_STR,raw_shell, sizeof raw_shell, NULL},
#ifdef USE_PGP
{"showpgppreamble",	-1L,DT_BOL,(char *)&pgp_noarmor, 0, NULL},
#endif
{"showto",		-1L,DT_BOL,(char *)&showto, 0, NULL},
{"sigdashes",		-1L,DT_BOL,(char *)&sig_dashes, 0, NULL},
{"signature",		-1L,DT_MLT,(char *)SIGS, 0, NULL},
{"sleepmsg",		-1L,DT_NUM,(char *)&sleepmsg, 0, NULL},
  /* {"softkeys",		-1L,DT_BOL|FL_OR,(char *)&hp_softkeys, 0, 
     NULL}, */
{"sort",		-1L,DT_SYN,"sortby", 0, NULL},
{"sortby",		-1L,DT_SRT,(char *)&sortby, 0, NULL},
{"store",		-1L,DT_SYN,"alwaysstore", 0, NULL},
{"timeout",		-1L,DT_NUM,(char *)&elm_timeout, 0, NULL},
{"titles",		-1L,DT_BOL,(char *)&title_messages, 0, NULL},
{"tmpdir",		-1L,DT_STR,raw_temp_dir, sizeof raw_temp_dir, NULL},
#ifdef USE_PGP
{"usepgppass",          -1L,DT_BOL,(char *)&pgp_keeppass, 0, NULL},
#endif
{"userlevel",		-1L,DT_NUM,(char *)&user_level, 0, &USERLEVEL},
{"username",		-1L,DT_SYN,"fullname", 0, NULL},
{"usetite",		-1L,DT_BOL|FL_AND,(char *)&use_tite, 0, NULL},
{"visualeditor",	-1L,DT_STR,v_editor, sizeof v_editor, NULL},
{"weed",		-1L,DT_BOL,(char *)&elm_filter, 0, NULL},
{"weedout",		-1L,DT_WEE,(char *)weedlist, 0, NULL},
};
int NUMBER_OF_SAVEABLE_OPTIONS=(sizeof(save_info_data)/sizeof(save_info_recs_init));
save_info_recs *save_info = (save_info_recs *) save_info_data;
#else
extern save_info_recs *save_info;
extern int NUMBER_OF_SAVEABLE_OPTIONS;
#endif

